import {Injectable} from '@angular/core';
import {ActivatedRouteSnapshot, CanActivate, CanLoad, Route, Router, RouterStateSnapshot} from '@angular/router';
import {Observable} from 'rxjs/Observable';

import {filter, map, mergeMap} from "rxjs/operators";
import {UserManagementService} from "../openaireLibrary/services/user-management.service";
import {LoginErrorCodes} from "../openaireLibrary/login/utils/guardHelper.class";
import {Session} from "../openaireLibrary/login/utils/helper.class";
import {properties} from "../../environments/environment";
import {StakeholderService} from "../openaireLibrary/monitor/services/stakeholder.service";
import {of} from "rxjs";

@Injectable()
export class AdminDashboardGuard implements CanActivate {

  constructor(private router: Router,
              private stakeholderService: StakeholderService,
              private userManagementService: UserManagementService) {
  }

  check(path: string, alias: string): Observable<boolean> | boolean {
    if (Session.isLoggedIn()) {
      const obs = this.userManagementService.getUserInfo(false).pipe(map(user => {
        if(user) {
          return this.stakeholderService.getStakeholder(alias).pipe(map(stakeholder => {
            if(stakeholder) {
              return (Session.isPortalAdministrator(user) ||
                Session.isCurator(stakeholder.type, user) || Session.isManager(stakeholder.type, stakeholder.alias, user));
            } else {
              return false;
            }
          }));
        } else {
          return of(false);
        }
      }), mergeMap( authorized => {
        return authorized;
      }));
      obs.pipe(filter(isManager => !isManager)).subscribe(() => {
        this.router.navigate(['/user-info'], {queryParams: {'errorCode': LoginErrorCodes.NOT_ADMIN, 'redirectUrl':path}});
      });
      return obs;
    } else {
      this.router.navigate(['/user-info'], {queryParams: {'errorCode': LoginErrorCodes.NOT_LOGIN, 'redirectUrl':path}});
      return false;
    }
  }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | boolean {
    return this.check(state.url, route.params.stakeholder);
  }
}
